//
//  Network.h
//  Neuron1
//
//  Created by Leslie Smith on Mon Jun 07 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "syndata.h"

// #defines for determining the max size of Synapses
#define FACTOR_INPUTNEURON 0.5
#define FACTOR_NONINPUTNEURON 0.5

// number of timesteps between subpool de-allocations
#define N_SUBPOOL 2500

typedef struct spike spiketype ;
typedef spiketype *spikearray ;

struct spike // output spike holding structure
{
	int neuron ;
	float timeofspike ;
} ;

typedef struct nparams neuronparams ;
typedef neuronparams *neuronparamsarray ;

struct nparams // holds values of parameters for nondefault neurons
{
	float zerolevel;
	float initphase;
	float tonicvalue;
	float threshold;
	float refperiod;
	float dissipation;
	float squarediss ;
	float minact ;
} ;

@interface Network : NSObject {
id RandomNumberGenerator ; /* random number generator */
id ViewObject ; // the view object that initialises the network
NSMutableArray *NeuronArray ; /* the neurons in the network */
NSMutableDictionary *SimParameters ;
NSMutableDictionary *NeuronParameters ;
NSMutableDictionary *NoiseParameters ;
NSMutableDictionary *InputSpikeInformation ;

spikearray inspikes ; // input spikes
int number_input_spikes ;
spikearray spikesout ; // spikes produced
int numspikes ; // number of spikes made
int maxtotalspikes ; // maximal total number of spikes recorded

int n_neuron ; // total no of neurons
int input_neuron; // number of input neurons
double duration ; // duration of simulation in seconds
double timestep ; // simulation delta
int simsteps ; // no of simulation steps

int keepInputs ;
STDPinfo adaptvalues ;
NSMutableArray * Synapses ; // synapse objects ;
}

- init: sender ; // init for this object needs to know about the View that calls it

- supplyParameters: (NSMutableDictionary *) SimParams: (NSMutableDictionary *) NeuronParams: 
	(NSMutableDictionary *) NoiseParams ;

- applySimParameters: (NSMutableDictionary *) SP ;
- applyNeuronParameters: (NSMutableDictionary *) NeP ;
- applyNoiseParameters: (NSMutableDictionary *) NoP ;
- displayDictionary: Dict1 ;
- setup: (int) maxspikes : (int) displayAll:  (NSMutableDictionary *) SP: (NSMutableDictionary *) NeP: (NSMutableDictionary *) NoP;
- (int) start ;
- displayDictionary: Dict1 ;
- (spikearray) returnSpikes ;
- initSetRng: Rng ;
- setSynapses: (NSString *)synapsefile ;
- setAdaptiveParams: (STDPinfo) adaptparams ;
- setInputSpikes: (NSString *)inputspikefile ;
- setSpikeInput: (NSString *)spikeinputfile ;
- setInputSpikesFromArchive: (NSString *)spikefilename ;
- displaySynapses: (NSTextView *)SynapseDisplay ;
- saveSynapses: (NSString *) savespikefile ;
- (int) getNumberOfInputNeurons ;
- (int) getNumberOfNeurons ;

@end
