
#import <appkit/appkit.h>
#import "InputNeuron.h"

#define DEFTHRESH 1.0
#define DEFDISS 0.0
#define DEFTIME 0.01
#define DEFMINACT 0.0
#define DEFREFPD 0.0 
#define MAXPRESYNSYNAPSES 25

@interface Neuron :  InputNeuron
{
id Rng ; /* random number generator */
float activation; /* V-like state variable */
float threshold ; /* theta in equations */
float dissipation ; /* gamma in equations */
float squaredissipation ; /* dissipation for activation ^ 2 */
// float timestep ; /* default timestep in seconds */
float initphase ; /* initial phase */
// int output ; /* normally 0, 1 for one time step */
float resetlevel ; /* reset value for activation */
float minact ; /* minimum permitted activation */
float refractperiod ; /* refractory period */
int intrefperiod, refractcount ; /* for timing refractory period */
float tonicinput ; /* tonic input */
int nonoise ;
float tonicnoise, threshnoise, activationnoise ;
// synapses
NSMutableArray * PreSynaptic ; // array of synapses on this neuron's "dendrites"
// NSMutableArray * PostSynaptic ; // array of synapses innervated by this neuron (declared in InputNeuron)
}

- init ;
- (void) dealloc ;
- initial: (float) newthresh: (float) newdiss: 
	(float) minactivation: (float) refpd ; /* initialise */
- initSetRng: rngid ; /* set up random number generator */
- (int) isInputNeuron ;
- setTimeStep: (float) stepsize; /* set timestep */
- setTonic: (float) tonicinput ;
- setPhase: (float) phasepoint ; /* set initial phase */
- setThreshold: (float) thresh ;
- setRefPeriod: (float) refpd ; 
- setDissipation: (float) diss ; 
- setSquareDissipation: (float) squarediss ; 
- setMinimumActivation: (float) minactivn ; 
- setZerolevel: (float) zerolevel ;


- resetNeuron ; /* set activation and output back to 0 */

- noNoise ; /* set up absence of noise */
- tonicNoise: (float) level ; /* set up fractional noise on tonic input */
- thresholdNoise: (float) level ; /* set up noise on threshold */
- activationNoise: (float) level ; /* set up noise on activation */

- (float) getActivation; /* returns activation level */
- (int) getOutput ; /* returns output */
- (float) getTimeStep ; /* returns timestep in use last time step */
- (float) getThreshold ; /* return threshold */
- (float) getPhase ; /* return phase */
- (float) getTonic; /* returns tonic input level */
- (float) getPeriod ; /* returns calculated period */
- (float) getRefPeriod ; /* get refractory period */
- (float) getDissipation ; /* get dissipation */
- (float) getSquareDissipation; /* get 2nd order dissipation */

- removeSynapsesAndTargets ; // remove all prior to setting them up
- addSynapse: Syn ; // add a synapse to the set that this neuron owns (is postsynaptic)
- addTarget: Syn ; // add a synapse to the set that this neuron innervates (is presynaptic)


/* run for 1 timestep */
/* returns output value */
- (int) runStep:  (int) otherinput : (double) actime: (int) stepno ;
- fire: (double) actime: (int) stepno ; // action on firing (internal only)
- (float) calcSynContribution: (double) actime: (int) stepno ; // additive synapses
- (float) calcMultSynContribution: (double) actime: (int) stepno ;  // multiplicative synapses

@end
