//
//  dynsynapse.h
//  Neuron1
//
//  Created by Leslie Smith on Sat Jul 31 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//
// subclass of synapse implementing a dynamic synapse
//
// implements a simple dynamic synapse with recharge rate recharge
// utilisation rate utilisation.
// uses the alpha function to determine synapse effect
// primary differences from synapse.h is in the effect of near-coincident presynaptic
// spikes

#import <Foundation/Foundation.h>
#import "synapse.h"


@interface dynsynapse : synapse {
	float lastspike ; // for last spike time, however long ago that was.
	float rechargerate ; // rate for recharging dynamic synapse
	float utilrate; // rate for using synapse on presynaptic spike occurrence
	float level ; // level in reservoir (between 0 and 1)	
	float w_effs[MAXCONCURRENTPRESYN] ; //  array of possibly depressed weights
	float w_eff; // effective (depressed) weight
	// for NN pre->post STDP system
	float lastusedspikew_eff ; // effective weight for last used spike
	// for Classic pre->post STDP system
	float lastusedspikew_effarray[MAXKEPTSPIKES] ;
}

- setRechargerate: (float) rate ;
- setUtilrate: (float)  rate ;
- (float) getRechargerate ;
- (float) getUtilrate ;
- (syndatatype) getSynapseData ; // returns whole structure describing synapse configuration
- applySTDPPresynaptically : (float) presyntime effectiveweight: (float) effweight ;

- presynFires: (float) t_fire : (int) stepfire; // spike arrives at t_fire (step stepfire)
- (float) getSynapticContribution: (double) current_time: (int) stepno ; 
- postsynFires: (float) t_fire: (int) stepfire ; // for resetting alpha functions and STDP adjustment

@end
