#import <math.h>
// #import <c.h>
#import <libc.h>
#import "odds.h"

#define MINUSINFINITY -32768

int findmaxmod(frametype frame, int framelength)
{
   int index, max, min, maxmod ;
   min = 1000; max = -1000 ;
   for (index = 0; index < framelength; index++)
   {
     if ((int) frame[index] < min) min = (int) frame[index] ;
     if ((int) frame[index] >max) max = (int) frame[index] ;
   }
   if (max > (-min) ) maxmod = max ; else maxmod = (-min) ;
   return maxmod ;
}

float floatfindmaxmod(float *frame, int framelength)
{
   int index;
   float max, min, maxmod ;
   min = 1000.0; max = -1000.0 ;
   for (index = 0; index < framelength; index++)
   {
     if ( frame[index] < min) min =  frame[index] ;
     if ( frame[index] > max) max =  frame[index] ;
   }
   if (max > (-min) ) maxmod = max ; else maxmod = (-min) ;
   return maxmod ;
}

void calcpoweights(int sampleperiod, float product, float *weights, int max) 
/* compute the array of weights for power convolution */
/* formula is Sigma k exp(-kt) */
{
int wno ;
float k, wtemp ;
    /* k in exp(-kt) s.t. k.sampleperiod = product */
    k = product/sampleperiod ;
    /* init weight array */
    for  (wno = 0; wno < max ; wno++) weights[wno] = 0.0; 
    for (wno = 0; wno < max; wno++)
      {
         wtemp = k * exp(- wno * k) ;
	 if (wtemp == 0.0) break ;
	 weights[wno] = wtemp ;
      }
}

void calcpoweights2(int nweights, float *weights)
/* simple mean over a number of periods */
{
  int i ;
  for (i=0;i<nweights;i++) weights[i] = 1.0/nweights ;
}

void convolve(float *weights, int wtlength, frametype frame, int flength, 
         float maxconv)
/* convolve the weights with the frame */
/* assume frame has 0's preceeding it */
{
int index ;
int i, offset ;
float newvalue, increment ;
   /* start at end, so we can write answers in directly */
   for (index = flength - 1; index >= 0; index--)
   {
      newvalue = 0.0 ;
      for (i = 0;i<wtlength;i++)
      {
        offset = (index - i) ;
	if (offset  < 0) increment = 0 ; /* before start */
	else increment = frame[offset] * weights[i] ;
        newvalue +=  increment ;
      }
      frame[index] = (int) (newvalue/ maxconv) ;
   }
}

float calcintercept(float ratio, float decay)
/* calculate where the onset/offset filter (exponential) meets the X axis */
{
  return(log(ratio)/(decay - (decay/ratio))) ;
}

float calcinterceptg(float ratio, float decay)
/* calculate where the onset/offset filter (gaussian) meets the X axis */
{
 /* original, error return(sqrt(log(ratio)/(2 * (decay * (ratio - 1))))) ; */
 return(sqrt(log(ratio)/(2 * (decay * (1 - (1 / ratio)) ) ) ) ) ;
}

void calconsweights(float k, float r, float *weights, int max, float sampint)
/* calculate the weights for the onset/offset calculation (exponential) */
/* equation is k(exp(-k.t) - (1/r)exp(-(k/r)t)) */
{
  int index ;
  for (index = 0; index < max; index++)
  {
    weights[index] = k * (exp (sampint * (-k * index)) - 
          (exp(sampint * (-(k * index)/ r))/r)) ;
  }
}

void calconsweightsg(float k, float r, float *weights, int max, float sampint)
/* calculate the weights for the onset/offset calculation (gaussian) */
/* equation is sqrt(k)(exp(-k.t.t) - (1/sqrt(r))exp(-(k/r)t*t)) */
{
  int index ;
  float timesq ;
  for (index = 0; index < max; index++)
  {
    timesq = (sampint * index) * (sampint * index) ;
    weights[index] = sqrt(k) * (exp(-k * timesq) - 
                                    (exp(-k * timesq/r)/sqrt(r))) ;
  }
}

void calconsweightDG(float sigma, float *weights, int max, float sampint)
/* calculate the onset weights for the Derivative of Gaussian filter */
/* equation is */
/* -(t/(pow(M_E,pow(t,2)/(2*pow(sigma,2)))*sqrt(2*M_PI)*pow(sigma,3))) */
{
  int index ;
  float time ;
  for (index = 0; index < max; index++)
  {
  time = (sampint * (index - max/2)) ;
    weights[index] =  -(time/(pow(M_E,pow(time,2)/(2*pow(sigma,2)))*sqrt(2*M_PI)*pow(sigma,3)));
  }
}


void intnorm( float k, float r,int nwts,  float *fwts, int *iwts, int maxwt)
/* normalise the convolution weights to a max value of nwts */
/* by linearly multiplying by maxwt/(k - (k/r)) */
{
  int index ;
  float multiplier, result ;
  multiplier = (float) maxwt /(k - (k/r)) ;
  for (index = 0; index < nwts; index++)
  {
    result = *fwts++ * multiplier ;
    *iwts++ = (int) result ;
  }
}

float mvioc( float k, float r, int srate)
/* computes \sigma_0^intercept k exp(-kt) - (k/r) exp(-(k/r)t) */
{
   int sno;
   float t ;
   float sampleperiod = (1.0 / (float) srate) ;
   float  val = 0 ;
   float intercept = calcintercept(r,k) ;
   int nsamples = (int) rint( (double) (intercept * srate)) ;
   for (sno = 0; sno <= nsamples; sno++)
   {
     t = sampleperiod * sno ;
      val += (k * exp( -k * t) - (k/r) * exp( -(k/r) * t));
   } 
   return val ;
}

float mviocg( float k, float r, int srate)
/* computes \sigma_0^intercept sqrt(k)exp(-k .x^2)-sqrt(k/r)exp((-k/r).x^2) */
{
   int sno;
   float t ;
   float sampleperiod = (1.0 / (float) srate) ;
   float  val = 0 ;
   float intercept = calcintercept(r,k) ;
   int nsamples = (int) rint( (double) (intercept * srate)) ;
   for (sno = 0; sno <= nsamples; sno++)
   {
     t = sampleperiod * sno ;
      val += (sqrt(k) * exp( -k * (t * t)) - sqrt(k/r) * exp( -(k/r) * (t*t)));
   } 
   return val ;
}

float mviocDG( float sigma)
/* computes */
/* -(t/(pow(M_E,pow(t,2)/(2*pow(sigma,2)))*sqrt(2*M_PI)*pow(sigma,3))) */
/* at t = sigma */
{
return
-(-sigma/(pow(M_E,pow(sigma,2)/(2*pow(sigma,2)))*sqrt(2*M_PI)*pow(sigma,3))) ;
}

int bucketthreshold(int *bucket, float threshold, int bucketsize)
/* compute the threshold of a complete bucket */
/* return +1 only if whole of bucket is above threshold */
/* return -1 only if whole of bucket is below -threshold */
/* otherwise return 0 */
{
  int i ;
  int pos, neg, zero ;
  pos = FALSE; neg = FALSE; zero = FALSE ;
  for (i=0;i<bucketsize;i++)
    if (bucket[i] < -threshold) neg = TRUE ;
    else if (bucket[i] > threshold) pos = TRUE ;
    else zero = TRUE ;
  if ( (pos) && (!neg) && (!zero)) return(+1) ;
  else if ((!pos) && (neg) && (!zero)) return(-1) ;
  else return(0) ;
}

int findmax(int *segment, int length)
/* finds the index in the segment of the largest value */
/* returns a sensible value even if all are MINUSINFINITY */
{
  int maxval, index, mindex ;
	mindex = 0 ; // unnecessary
  maxval = MINUSINFINITY - 10 ; /* ! */
  for (index=0; index<length;index++)
    if (segment[index] > maxval)
      {
        mindex = index ;
	maxval = segment[index] ;
       }
   return(mindex) ;        
}

void peakfind(int *orig, int *peak, int length)
/* set each element of peak to 1 if a peak exists at that point in orig */
/* and to 0 if not. Only consider length values */
{
  int i,j,k, maxindex ;
  
  /* initialise peak to 0 */
  for (i=0;i<length;i++) peak[i] = 0 ;
  /* look for maxima */
  while (orig[(maxindex = findmax(orig, length))] != MINUSINFINITY)
  {
    peak[maxindex] = 1 ;
    /* find ends of the rivershed of this maximum */
    i = maxindex ;
    while (( i >= 1) && (orig[i-1] <= orig[i]))  {i-- ;} ;
    j = maxindex ;
    while ((j < (length-1)) && (orig[j+1] <= orig[j])) j = j+1 ;
    /* reset whole rivershed to MINUSINFINITY */
    for (k=i;k<=j;k++) orig[k] = MINUSINFINITY ;
  }
}

/* resample modified 24 Jan 1996 */
/* uses random no generator to do 1-dimensional dithering */
int resample(int insrate, /* input sample rate */
	int *indata, /* input data string */
	int inlength, /* input data length */
	int outsrate, /* output sample rate */
	int *outdata) /* output data string */
/* resamples string at *indata changing sample rate */
/* returning new data string length */
{
int bsize, /* bucket size*/
 nbuckets; /* number of buckets */
int index, bindex ;
int *inptr, *outptr;
int used, sampsinbucket ;
float exactbsize, bucketerror ; /* theoretical bucket size */
float frandom ;

/* calculate bucket size etc. */
exactbsize = ((float)insrate )/ (float) outsrate ;
bsize = floor(insrate/outsrate) ;
bucketerror = exactbsize - bsize ;
nbuckets = floor((inlength * outsrate) / insrate) ;

/* initialise random number generator */
   srandom(1) ;
   
/* printf("bsize = %d nbuckets = %d exactbsize = %f bucketerror = %f\n", 
   bsize, nbuckets,
   exactbsize, bucketerror) ; */

/* now do the resampling itself */
used = 0 ;

inptr = indata ;
outptr = outdata ;
for (index = 0; index < nbuckets; index++)
{
  /* sum up one bucket */
  sampsinbucket = bsize ;
  *outptr = 0 ;
  for (bindex=0; bindex < bsize; bindex++)
    *outptr = *outptr + *inptr++ ;
  used = used + bsize ;
  /* add an extra one sometimes */
  if (bucketerror != 0)
  {
  /* generate a random number between 0 and 1 */
    frandom = ((float)(random() % 10000)) / 10000.0 ;
    if (frandom <= bucketerror)
    {
      *outptr += *inptr++ ;
      sampsinbucket++ ;
      used++;
    }
  }
  *outptr = *outptr / sampsinbucket ; /* normalise */
  outptr++ ;
}
return(nbuckets) ;
}

float ERB( float f)
/* calculate erb: Moore and Glasberg 1983*/
{
return(24.7 * ((4.37 * f)/1000.0 + 1 ) );
}

float MFB(float cf, float bwmin, float quality)
/* calculate minimum filter bandwidth: follows change in AIM, cf p2, AAM HAP */
/* problems and omissions */
/* equivalent to ERB when bwmin=24.7, and quality = 9.265 */
{
return(bwmin + cf/quality) ;
}


