
#import "Neuron.h"
#import <math.h>
#import "Random.h"

@implementation Neuron

- init
/* initialise on setup */
{
	[super init] ;
	[self initial: DEFTHRESH: DEFDISS: DEFMINACT: DEFREFPD] ;
	tonicnoise = 0 ;
	threshnoise = 0 ;
	return self ;
}

- initial: (float) newthresh: (float) newdiss : 
           (float) minactivation: (float) refpd  /* initialise */
{
	threshold = newthresh ;
	dissipation = newdiss ;
	minact = minactivation ; 
	refractperiod = refpd ;
	activation = 0.0 ;
	timestep = DEFTIME ;
	output = 0 ;
	return self ;
}

- initSetRng: rngid
/* set up random number generator class object */
{
	Rng = rngid ;
	return self ;
}

- setTimeStep: (float) stepsize  /* set timestep */
/* must be after setting of refractory period */
{
	timestep = stepsize ;
	if (refractperiod == 0) intrefperiod = 0 ;
	else intrefperiod = ceil(refractperiod/timestep) ;
	return self ;
}

- setRefPeriod: (float) refpd 
{
	refractperiod = refpd ;
	return self ;
}

- setTonic: (float) tonic /* tonic input */
{
	tonicinput = tonic ;
	return self ;
}

- setThreshold: (float) thresh 
{
	threshold = thresh ;
	return self ;
}

- setDissipation: (float) diss
{
	dissipation = diss ;
	return self ;
}

- setPhase: (float) phasepoint  /* set initial phase */
{
/* since activation(t) = */
/*                (tonicinput/dissipation)(1 - exp(- dissipation * t)) */
/* and the period, P, can be calculated, we can compute */
/* activation(t) = above, with period*phasepoint to be the activation */
/* at this phase in the cycle */
/* N.B. 0<= phasepoint < 1 */
/* If tonic is 0, then don't use getPeriod! */
/* instead, set activation to phase fraction of thereshold (LSS 4.4.2004) */
	initphase = phasepoint ;
	if (tonicinput == 0) 
		activation = phasepoint *  threshold ;
	else
	{
	if (dissipation !=0)
	 activation =  (tonicinput/dissipation) *
	 (1 - exp(- dissipation * (phasepoint * [self getPeriod]))) ;
	 else activation = tonicinput * (phasepoint * [self getPeriod]) ;
	}
	return self ;
}

- resetNeuron
{
	output = 0 ; 
	return self ;
}

- noNoise
{
	nonoise = YES ;
	return self ;
}

- tonicNoise: (float) level 
/* set up fractional noise on tonic input */
{
	nonoise = NO ;
	tonicnoise = level ;
	return self ;
}

- thresholdNoise: (float) level 
/* set up noise on threshold */
{
	nonoise = NO ;
	threshnoise = level ;
	return self ;
}

- activationNoise: (float) level 
/* set up noise on threshold */
{
	nonoise = NO ;
	activationnoise = level ;
	return self ;
}

- (float) getActivation  /* returns activation level */
{
	return activation ;
}

- (float) getRefPeriod
{
	return refractperiod ;
}

- (int) getOutput   /* returns output */
{
	return output ;
}

- (float) getTimeStep 
{
	return timestep ;
}

- (float) getDissipation
{
	return dissipation ;
}

- (float) getThreshold 
{ 
	return threshold ;
}

- (float) getTonic
{
	return tonicinput ;
}

- (float) getPeriod
/* calculate and return period */
{
float period ;
/* assumes constant tonic input and nothing else! */
	if (dissipation == 0)
		period = threshold/tonicinput ;
	else
		period = (1.0 / dissipation) *
			log(tonicinput/(tonicinput - 
				(dissipation * threshold))) ;
	/* add refractory period */
	period += refractperiod ;
	return period ;
}

- (float) getPhase
{
	return initphase ;
}

/* run for 1 timestep */
/* returns output value */
- (int) runStep:   (float) otherinput 
{
float tonic, thrshld ;
  if (refractcount != 0)
  {
  /* in refractory period */
   refractcount-- ;
   output = 0;
   activation = 0 ;
  }
  else
  {
  
	if (nonoise)
	{
// Note: should add the tonic first, then if that causes firing
// fire. Then add the Other input (and if that causes firing fire)
// and take the final output...
/* add tonic first */
		activation = activation  + 
		timestep * (tonicinput - 
			(dissipation * activation)) ;
		/* does it fire? */	
		if (activation >= threshold)
		{ /* set output */
		 output = 1; 
		 /* set refractory count */
		 refractcount = intrefperiod ;
		 /* reset activation to the inter-neuron input */
	 	 if (refractcount == 0) activation = otherinput ;
		 else activation = 0 ;

		}
	 	else
		{
		 activation = activation + otherinput ;
		 if (activation >= threshold)
		  { /* set output */
		   output = 1; 
		   /* reset activation to the inter-neuron input */
	 	   activation = 0 ;
		   /* set refractory count */
		   refractcount = intrefperiod ;
		  }
		  else output = 0 ;
		}
		/* check minimum activation not exceeded */
		if (activation < minact) activation = minact ;

	}
	else
	{
		/* add tonic noise */
		if (tonicnoise != 0)
		tonic = (tonicinput * 
		         (1 - ([Rng plusminus] * tonicnoise)));
			 else tonic = tonicinput ;

		/* add threshold noise */
		if (threshnoise != 0) 
		  thrshld = (threshold * (1 + [Rng plusminus] * threshnoise));
		  else thrshld = threshold ;	

		/* calc activation: add tonic first */
		activation +=  timestep * (tonic
		 	- (dissipation * activation))   ;
		/* add activation noise */
		 if (activationnoise != 0)
		 activation = (activation * 
		 	(1 - ([Rng plusminus] * activationnoise))) ;
		/* does it fire? */
		if (activation >= thrshld)
		{ /* set output */
		 output = 1; 
		 /* set refractory count */
		 refractcount = intrefperiod ;
		 /* reset activation */
		 if (refractcount == 0) activation = otherinput ;
		 else activation = 0 ;

		}
	 	else
		{
			activation += otherinput ;
			if (activation >= thrshld)
			{ /* set output */
		 	  output = 1; 
		 	  /* reset activation */
	 	 	  activation = 0 ;
		 	  /* set refractory count */
			   refractcount = intrefperiod ;
			}
			else output = 0 ;
		}
		/* check minimum activation not exceeded */
		if (activation < minact) activation = minact ;

	}
	}
	return output ;
}

@end
