//
//  synapse.m
//  Neuron1
//
//  Created by Leslie Smith on Tue Jun 15 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "synapse.h"
// #import <CoreServices/fp.h>


@implementation synapse

- init
	/* initialise on setup */
{
	[super init] ;
	neverspiked = YES ; 
	presynspikes = 0 ; // show that no spikes have been received
	delay = 0 ;
	delaysteps = 0 ; // default is no delay
	resetContributionOnInputSpike = NO ; 
	lastusedspiketime = -1 ;
	lastusedspikestep = -1000 ; // init to impossible values that will result in no effect
	lastpostsyntime = -1 ;
	lastpostsynstep = -1000 ;
	lastusednumber = 0 ; // number of presynaptic spikes recently used
	return self ;
}

- setWeight:  (float) w 
{
	weight = w ;
	return self ;
}

- setAlpha: (float) alph alphareset: (bool) alphareset 
{ 
	alpha = alph ;
	alpha_reset = alphareset ;
	return self ;
}

- setSynapseType:  (int)s 
{
	synapsetype = s ;
	return self ;
}

- setPresynNeuron: (Neuron *) NNN number: (int) preno
{
	PresynNeuron = NNN ;
	prenumber = preno ; // for use in saving synapses to file
	return self ;
}

- setPostsynNeuron: (Neuron *) NNN number: (int) postno
{
	PostsynNeuron = NNN ;
	postnumber = postno ; // for use in saving synapses to file
	return self ;
}
	
- setDelay: (double)d : (int) dsteps 
{
	delay = d ;
	delaysteps = dsteps ;
	return self ;
}

- setResetContributionOnInputSpike: (BOOL) resetval 
// YES implies only 1 spike contribute
{
	resetContributionOnInputSpike = resetval ;
	return self ;
}

- setAdaptive: (BOOL) adaptive: (STDPinfo) adaptparams
{
	// NB Must not be called till weight has been set
	isadaptive = adaptive ;
	if (isadaptive) // set up the parameters for adaptation if this synapse is adaptive
	{
		adaptvalues = adaptparams ; 
		// change the sign of the weight changes if antihebbian for inhibitory has been selected
		// and the initial weight is inhibitory
		if ((adaptvalues.antihebbianforinhibitory) && (weight < 0))
		{
			adaptvalues.prepostDeltaWeight = - adaptparams.prepostDeltaWeight ;
			adaptvalues.postpreDeltaWeight = - adaptparams.postpreDeltaWeight ;
		}
		// for some combinations we need to know if the synapse is exc or inh (specifically when zxpermitted = NO)
		if (weight >= 0) adaptvalues.excitatory = YES ;
		else adaptvalues.excitatory = NO ;
	}
	return self ;
}

- (float) getWeight
{
	return weight ;
}

- (int) getSynapseType 
{
	return synapsetype ;
}

- getPresynNeuron
{
	return PresynNeuron ;
}

- getPostsynNeuron
{
	return PostsynNeuron ;
}

- presynFires: (float) t_fire: (int) stepfire
// notification that presynaptic neuron has fired
// updated to allow multiple presyn spikes LSS 4 Oct 2004
// updated to include delays 7 Oct 2004 LSS
{
	int i ;
	neverspiked = NO ;
	
	// presynaptic-based adaptivity would go here if there were no delays
	
	
	if (resetContributionOnInputSpike)
		presynspikes = 1 ; // always use only 1
		else presynspikes = presynspikes + 1 ;
	if (presynspikes > MAXCONCURRENTPRESYN)
	{
		NSLog(@"Synapse.presynFires: Too many presynaptic spikes (%i). Alter MAXCONCURRENTPRESYN \n",
			  presynspikes) ;
		return self ;
	}
	// add this spike to the lastspiketimes and lastspikesteps arrays
	// always keep most recent spike at index 0
	// shift rest along  before storing this one
	for (i=presynspikes-1;i>0;i--)
	{
		lastspiketimes[i] = lastspiketimes[i-1] ;
		lastspikesteps[i] = lastspikesteps[i-1]  ;
	}
	lastspiketimes[0] = t_fire + delay ;
	lastspikesteps[0] = stepfire + delaysteps ;

	// printf("%s%f%s","Synapse:presynFires spike at ", t_fire, "\n") ;
	return self ;
}

- computeSynapseEffects: (double) timestep 
{
	// Note: over-ridden in ShuntSynapse object: danger
	// compute the value of the alpha function for each timestep
	// then place these values into an array for use by getSynapticContribution
	// uses alpha function
	// (alpha * alpha) * t * exp(-alpha * (t)) ;
	// addition: if alpha = 0, then make whole effect occur at next timestep
	// use this for synapses from input neurons
	int i ;
	double total ;
	float upper, lower ;
	if (alpha == 0) // special: alpha == 0 for synapses from input units
	{
		lengthalphaeffect = 2 ;
		alphaarray[0] = 0 ;
		alphaarray[1] = 1 ;
		return self ;
	}
// normal situation
	// calculate the alpha function for 10 * 1/alpha microsteps
	// now parcel them up for use by getSynapticContribution
	total = 0 ;
	lengthalphaeffect = (int) ceil((10.0/alpha) * (1/timestep)) ;
	if (lengthalphaeffect > MAXTIMESTEPEFFECT)
		printf("%s","synapse:computeSynapseEffects. Overlong effect of alpha function. Increase MAXTIMESTEPEFFECT \n");

	alphaarray[0] = 0 ;
	for (i=1;i<lengthalphaeffect;i++)
	{ // integrate alpha function and store result
		upper = - alpha * exp(-alpha * (i*timestep))*((i*timestep) + (1/alpha)) ;
		lower = - alpha * exp(-alpha * ((i-1)*timestep))*(((i-1)*timestep) + (1/alpha)) ;
		alphaarray[i] = (upper - lower) ;
		total = total + alphaarray[i] ;
	}
	// normalise values	
	for (i=1;i<lengthalphaeffect;i++)
		alphaarray[i] = alphaarray[i]/total ;
	// temp
	// for (i=0;i<lengthalphaeffect;i++)
	//  	printf("%s%i%f%s", "alphaeffect ", i, alphaarray[i], "\n") ;
	return self ;
}

- (float) getSynapticContribution: (double) current_time: (int) stepno
{
	// Note: over-ridden in ShuntSynapse object: danger	
	// updated 7 Oct 2004 to include delays
	float contribution;
	int timesincespike ;
	int i,j, deduct ;
	if ((neverspiked) || (presynspikes == 0))
	{
		contribution = 0 ;
		return contribution ;
	}
	if (do_reset_alpha) // not used
	{
		// remove those presynaptic inputs which have arrived
		// but not those in the delay (i.e. in presynaptic axon)
		for (i=presynspikes-1;i>=0;i--)
			if ((stepno - lastspikesteps[i]) > 0)
				presynspikes-- ;
		contribution = 0 ;
		do_reset_alpha = NO ; // turn off alpha resetting here.
	}
	else
	{
		// add up contribution of all presynaptic spikes which have actually arrived
		contribution = 0 ;
		for (i=0; i< presynspikes; i++)
		{
			timesincespike = (stepno - lastspikesteps[i]) ;
			if (isadaptive && (timesincespike == 0))
				[self applySTDPPresynaptically : lastspiketimes[i]] ;
			if ((timesincespike >= 0) && (timesincespike < lengthalphaeffect)) // ignore if negative: in delay
				contribution = contribution + weight * alphaarray[stepno - lastspikesteps[i]] ; 
		}
		// delete presyn spikes that have come to an end
		// if 
		// there may be > 1	

		while ((presynspikes > 0) && ((stepno - lastspikesteps[presynspikes - 1]) >= lengthalphaeffect))
		{			
			presynspikes = presynspikes - 1 ;
			// update history information used for adaptive synapses
			if (isadaptive)
			{
				lastusedspiketime = lastspiketimes[presynspikes] ;
				lastusedspikestep = lastspikesteps[presynspikes] ; // stores most recent last spike
				// keep a longer history too, used for pre->post classic STDP 
				// slide history up, deleting any which are too old
				deduct = 0 ;
				for (j=lastusednumber;j>0;j--)
				{
					// only keep them if they are within tikme relevant to pre->post classic STDP
					// bug fix 22012008 (MAXPRESYNDT * adaptvalues.prepostTau) replaces (MAXPRESYNDT / adaptvalues.prepostTau)
					if ( (current_time - lastusedspiketimearray[j-1]) < (MIN(0.1, MAXPRESYNDT / adaptvalues.prepostTau)))
					{
						lastusedspiketimearray[j] = lastusedspiketimearray[j-1] ;
						lastusedspikesteparray[j] = lastusedspikesteparray[j-1] ;
					}
					else deduct++ ;
				}
				lastusednumber = lastusednumber - deduct ;
				lastusedspiketimearray[0] = lastusedspiketime ;
				lastusedspikesteparray[0] = lastusedspikestep ;
				if (lastusednumber == MAXKEPTSPIKES - 1)
					NSLog(@"Synapse:getSynapticContribution run out of space in lastusedspiketimearray. %i\n",
						  lastusednumber) ;
				else
				lastusednumber++ ;
			}
				
		}

	}
	
	return contribution;
}

- applySTDPPresynaptically : (float) presyntime
// compute and apply post=>pre STDP LTD
{	float oldweight = weight ; // weight before change
	
	if (lastpostsyntime < 0) return self ;
	if (adaptvalues.LTPwinner && postsynusedadaptively) return self ;
	if ((lastpostsyntime - presyntime) > (MAXPOSTSYNDT / adaptvalues.postpreTau))
		return self ;
	if (adaptvalues.postpreNN && postsynusedpresynaptically) return self ;
	// if this synapse is excitatory, and zero-crossing is not permitted, and the weight is 0, leave it there.
	if (adaptvalues.excitatory && !adaptvalues.zxpermitted && (weight == 0)) return self ;
	// if this one is inhibitory, and zero-crossing is not permitted, and this one is antihebbian (so LTP)
	// and the weight is 0, leave it there.
	if (!adaptvalues.excitatory && !adaptvalues.zxpermitted && 
		adaptvalues.antihebbianforinhibitory && (weight == 0)) return self ;
	// go ahead and alter weight
	if (adaptvalues.postpreadditive)
		weight = weight - adaptvalues.postpreDeltaWeight * 
			exp(- fabs((lastpostsyntime - presyntime)) * adaptvalues.postpreTau) ;
	else // multiplicative
		 // note that weight must be < 1
		weight = weight - (adaptvalues.postpreDeltaWeight * 
						   exp(- fabs((lastpostsyntime - presyntime)) * adaptvalues.postpreTau) * weight) ;
	if (!adaptvalues.zxpermitted) // weight may not change sign
	{
		if (((oldweight < 0) && (weight >= 0)) ||
			((oldweight > 0) && (weight <= 0 ))) // NB can be going either way here
		{
			weight = 0 ;
			if (adaptvalues.zeroabsorbing)
				[self setAdaptive: NO : adaptvalues] ; // absorbing boundary
		}
	}	
	// check weight not above (antihebbian) boundary
	if (weight > adaptvalues.maxallowedweight) weight = adaptvalues.maxallowedweight ;	
	// and not below either 
	if (weight < adaptvalues.minallowedweight) weight = adaptvalues.minallowedweight ;		
		
	postsynusedpresynaptically = YES ;
				
	return self ;
}

- postsynFires: (float) t_fire: (int) stepfire
	// Use for resetting alpha, and (later) for STDP
{
	float dt = 0 ;
	float oldweight=weight ;
	float alphamult ;
	int i, ii ;
	int validpresynspikes ;
	float dtsforadaptation[MAXCONCURRENTPRESYN+MAXKEPTSPIKES] ; // absolute possible max size
																// record time of last spike
	lastpostsyntime = t_fire ;
	lastpostsynstep = stepfire ;
	postsynusedpresynaptically = NO ;
	postsynusedadaptively = NO ;
	// default is reset excitatory synapses, but not inhibitory ones
	if (alpha_reset) do_reset_alpha = YES  ; // ignore previous spikes
	if (isadaptive)
	{
		// adaptivity goes here. (LTP unless antihebbian)
		// if this synapse is inhibitory, and zero-crossing is not permitted, and the weight is 0, leave it there.
		if (!adaptvalues.excitatory && !adaptvalues.zxpermitted && (weight == 0)) return self ;
		// if this one is excitatory, and zero-crossing is not permitted, and this one is antihebbian (so LTD)
		// and the weight is 0, leave it there.
		if (adaptvalues.excitatory && !adaptvalues.zxpermitted && 
			adaptvalues.antihebbianforinhibitory && (weight == 0)) return self ;
		
		// Otherwise, if there are spikes to be processed, and the most recent one has actually arrived,
		// or if there were spikes some time ago
		if (((presynspikes > 0) && ((t_fire - lastspiketimes[presynspikes-1]) > 0)) || (lastusedspiketime > 0)) 
			// < 0 implies no presynaptic spikes
		{
			if (adaptvalues.prepostNN)
			{
				// find the time since last spike
				if ((presynspikes > 0) && ((t_fire - lastspiketimes[presynspikes-1]) > 0))
				{ // check: there may be a number of spikes currently inside the alpha function
					ii = presynspikes-1 ;
					while ((ii >= 0) && ((t_fire - lastspiketimes[ii]) > 0))
					{
						dt = (t_fire - lastspiketimes[ii]) ;
						ii = ii - 1 ;
					}
				}
				else dt = (t_fire - lastusedspiketime)  ;
				if (dt < MAXPRESYNDT / adaptvalues.prepostTau) // do nothing if time since last presynaptic spike was too long
				{
					if (adaptvalues.prepostadditive)
					{
						if (alpha != 0)
							weight = weight + adaptvalues.prepostDeltaWeight * 
								exp(- fabs(dt - (1/alpha)) * adaptvalues.prepostTau) ;
						else weight = weight + adaptvalues.prepostDeltaWeight * 
							exp(- fabs(dt) * adaptvalues.prepostTau) ;
					}
					else // multiplicative
						 // note that weight must be < 1
					{
						if (alpha != 0)
							weight = weight + (adaptvalues.prepostDeltaWeight * 
										   exp(- fabs(dt - (1/alpha)) * adaptvalues.prepostTau) * (1-weight)) ;
						else weight = weight + (adaptvalues.prepostDeltaWeight * 
												exp(- fabs(dt) * adaptvalues.prepostTau) * (1-weight)) ;
					}
					postsynusedadaptively = YES ;
					if (!adaptvalues.zxpermitted) // weight may not change sign
					{
						if (((oldweight < 0) && (weight >= 0)) ||
							((oldweight > 0) && (weight <= 0 ))) // NB can be going either way here
						{
							weight = 0 ;
							if (adaptvalues.zeroabsorbing)
							[self setAdaptive: NO : adaptvalues] ; // absorbing boundary
						}
					}	
					// check weight not above boundary
					if (weight > adaptvalues.maxallowedweight) weight = adaptvalues.maxallowedweight ;
					// and not below either (antihebbian could cause this)
					if (weight < adaptvalues.minallowedweight) weight = adaptvalues.minallowedweight ;	
				}
			} //  (adaptvalues.prepostNN)
			else // pre -> post classic
			{
				// needs to consider both spikes still within their alpha function (in lastspiketimes)
				// and spikes whose alpha function has finished (in lastusedspiketimearray)
				// find the time since last spikes
				validpresynspikes = 0 ; // use to count the presaynaptic spikes helping this Dw
				if ((presynspikes > 0) && ((t_fire - lastspiketimes[presynspikes-1]) > 0))
				{ // check: there may be a number of spikes currently inside the alpha function
					ii = presynspikes-1 ;
					while ((ii >= 0) && ((t_fire - lastspiketimes[ii]) > 0))
					{
						dt = (t_fire - lastspiketimes[ii]) ;
						dtsforadaptation[validpresynspikes] = dt ;
						validpresynspikes++ ;
						ii = ii - 1 ;
					}
				}
				// now find those whose alpha function has ended
				for (i=0;i<lastusednumber;i++)
				{
					dtsforadaptation[validpresynspikes] = (t_fire - lastusedspiketimearray[i]) ;
					validpresynspikes++ ;
				}
				// now do the actual weight update
				for (i=0;i<validpresynspikes;i++)
				{
					// bug fix 23 Jan 2005
					if (alpha == 0) alphamult = 0 ;
					else
						alphamult = (1/alpha) ;
					if (dt < MAXPRESYNDT / adaptvalues.prepostTau)
					{
						if (adaptvalues.prepostadditive)
							weight = weight + adaptvalues.prepostDeltaWeight * 
								exp(- fabs(dtsforadaptation[i] - alphamult) * adaptvalues.prepostTau) ;
						else // multiplicative
							 // note that weight must be < 1
							weight = weight + (adaptvalues.prepostDeltaWeight * 
											   exp(- fabs(dtsforadaptation[i] - alphamult) * adaptvalues.prepostTau) * (1-weight)) ;
					}
				}
				postsynusedadaptively = YES ;
				if (!adaptvalues.zxpermitted) // weight may not change sign
				{
					if (((oldweight < 0) && (weight >= 0)) ||
						((oldweight > 0) && (weight <= 0 ))) // NB can be going either way here
					{
						weight = 0 ;
						if (adaptvalues.zeroabsorbing)
						[self setAdaptive: NO : adaptvalues] ; // absorbing boundary
					}
				}	
				// check weight not above boundary
				if (weight > adaptvalues.maxallowedweight) weight = adaptvalues.maxallowedweight ;
				// and not below either (antihebbian could cause this)
				if (weight < adaptvalues.minallowedweight) weight = adaptvalues.minallowedweight ;					
			}
		}
	}
	return self ;
}

- (syndatatype) getSynapseData 
{
	syndatatype syn1 ;
	int i1 ;
	// populate syn1 
	i1=0 ;
	syn1.synapsetype[i1++] = 's'  ;	
	if (resetContributionOnInputSpike)
		syn1.synapsetype[i1++] = 'r' ;
	if (isadaptive) 
		syn1.synapsetype[i1++] = 'a' ;

	syn1.synapsetype[i1] = '\0' ; // terminator
	syn1.presyn = prenumber ; 
	syn1.postsyn = postnumber ; 
	syn1.syntype = synapsetype ;
	syn1.weight = weight ;
	syn1.alpha = alpha ;
	syn1.delay = delay ;
	syn1.rechargerate = 0 ; // fill in depsyn
	syn1.utilrate = 0 ; // fill in depsyn
	return syn1 ;
}

@end
